@@ -733,112 +733,41 @@ def rolling_allocation(f, X, n=252, s=63, ret=True, drift=True, filtering_thresh
 
     return portfolio, w_mat
 
-def rolling_ensembling(X, w_mat_predictors, w_mat_target, n=252, s=63, model_description='equireparti',ret=True, drift=True, **kwargs):
+
+def rolling_ensembling(X, w_mat_predictors_list, w_mat_target, n=252, s=63,ret=True,  **kwargs):
     X = pd.DataFrame(X).fillna(method='ffill')
     if ret:
         X_ = X.pct_change()
     else:
         X_ = X
-    w_mat_predictors = pd.DataFrame(w_mat_predictors).fillna(0)
-    idx = w_mat_predictors.index
+
+    assert len(w_mat_predictors_list) > 1
+    to_apply = lambda my_df : pd.DataFrame(my_df).fillna(0)
+    w_mat_predictors_list = list(map(to_apply, w_mat_predictors_list))
+
+    idx = w_mat_predictors_list[0].index
     w_mat = pd.DataFrame(index=idx, columns=w_mat_target.columns)
     portfolio = pd.Series(100., index=idx, name='portfolio')
-
     roll = rolling._EnsemblingRollingMechanism(idx, n=n, s=s)
 
-    def ensemble_process(series):
-        # True if less than 50% of obs. are constant
-        return series.value_counts(dropna=False).max() < 0.5 * n
-    print('n '+str(n))
-    print('s '+str(s))
-    print('model' + model_description)
     for slice_n, slice_s in roll():
-        # Select X
-        print('slice_n')
-        print(slice_n)
-        print('slice_s')
-        print(slice_s)
-        print('roll_d')
-        print(roll.d)
 
-        sub_X = X_.loc[slice_n].copy()
-        #assets = list(X.columns[sub_X.apply(ensemble_process)])
-        assets = list(X.columns)
+        train_ret = X_.loc[slice_n].copy()
+        train_forward_looking = w_mat_target.loc[slice_n].copy()
+        test_forward_looking = w_mat_target.loc[slice_s].copy()
 
-        train_X = w_mat_predictors.loc[slice_n].copy()
-        train_y = w_mat_target.loc[slice_n, assets].copy()
+        to_train = lambda my_df: my_df.loc[slice_n].copy()
+        to_test = lambda my_df: my_df.loc[slice_s].copy()
 
-        test_X = w_mat_predictors.loc[slice_s].copy()
-        test_y = w_mat_target.loc[slice_s, assets].copy()
-        ensembled_weights = {}
-        for weightToModel in assets:
-            #mask = [col for col in train_X.columns if weightToModel in col]
-            mask = [col for col in train_X.columns]
+        train_weights = list(map(to_train, w_mat_predictors_list))
+        test_weights = list(map(to_test, w_mat_predictors_list))
 
-            preds = train_X[mask]
-            target = train_y[weightToModel]
-            if model_description == 'gradient_boost':
-                model = GradientBoostingRegressor(loss='quantile',
-                                                  n_estimators=250, max_depth=3,
-                                                  learning_rate=.001, min_samples_leaf=9,
-                                                  min_samples_split=9)
-                model.fit(preds, target)
-                pred_y_grad_boost = model.predict(test_X[mask])
-                pred_y = pred_y_grad_boost
-            elif model_description == 'linear_regression':
-                reg = LinearRegression().fit(preds,target)
-                pred_y_reg_lin = reg.predict(test_X[mask])
-                pred_y = pred_y_reg_lin
-            elif model_description == 'equireparti':
-                mask = [col for col in train_X.columns if weightToModel in col]
-                pred_y_mean = test_X[mask].mean(axis = 1)
-                pred_y = pred_y_mean
-            elif model_description == 'random_forest':
-                clf = RandomForestClassifier(n_estimators=100, max_depth=2,random_state = 0)
-                clf.fit(preds, target)
-                pred_y_rand_forest = clf.predict(test_X[mask])
-                pred_y = pred_y_rand_forest
-            # print("reg lin")
-            # print(pred_y_reg_lin)
-            # print("mean")
-            # print(pred_y_mean)
-            # print("best")
-            # print(test_y[weightToModel])
-
-            # print("pred_y")
-            # print(pred_y.sum().sum())
-            # print("target")
-            # print(target.sum().sum())
-            ensembled_weights[weightToModel] = pred_y
-            ### end of modeling steps
-
-        ensembled_weights_df = pd.DataFrame.from_dict(ensembled_weights)
-        ensembled_weights_df_bis = pd.DataFrame(index=ensembled_weights_df.index, columns=ensembled_weights_df.columns)
-        for index, row in ensembled_weights_df.iterrows():
-            if row.sum() == 0:
-                row = row + 0.00000001
-            if np.min(row) < 0:
-                row = row - np.min(row)
-            row = (row) / row.sum()
-            ensembled_weights_df_bis.loc[index] = row
-
-        # ### renormalizing the weights
-        # print(w[assets].sum())
-        # if w[assets].sum() == 0:
-        #     w[assets] = w[assets] + 0.0000000001
-        #
-        # if np.min(w) < 0:
-        #     w = w - np.min(w)
-        # w = (w) / w.sum()
-        # w_mat.loc[roll.d, :] = w
-
-        # w_mat.loc[slice_s, assets] =ensembled_weights_df[assets].values
-        ensembled_weights_df_bis = ensembled_weights_df_bis.fillna(0)
-        w_mat.loc[slice_s, assets] = ensembled_weights_df_bis.values
+        x = metrics.find_best_sharpe(train_weights, train_ret)
+        pred_y = metrics.compute_average_prediction(x, test_weights)
+        pred_y = pred_y.fillna(0)
+        w_mat.loc[slice_s] = pred_y.values
         #w_mat.loc[ensembled_weights_df_bis.index, assets] = ensembled_weights_df_bis.values
-
         w = w_mat.loc[roll.d, :].fillna(0.)
-
         # Compute portfolio performance
         perf = _perf_alloc(
             X.loc[slice_s, :].fillna(method='bfill').fillna(0.00000000001).values,
@@ -846,10 +775,10 @@ def rolling_ensembling(X, w_mat_predictors, w_mat_target, n=252, s=63, model_des
             drift=True
         )
         portfolio.loc[slice_s] = portfolio.loc[roll.d] * perf.flatten()
-
     w_mat = w_mat.fillna(method='ffill').fillna(0.)
-
     return portfolio, w_mat
+
+
 # =========================================================================== #
 #                                   Tools                                     #
 # =========================================================================== #
